package cz.muni.pa165.movie;

import cz.muni.pa165.core.exception.ResourceNotFoundException;
import cz.muni.pa165.movie.data.repository.MovieRepository;
import cz.muni.pa165.movie.service.MovieService;
import java.util.List;
import java.util.Optional;
import org.junit.jupiter.api.BeforeEach;
import org.mockito.Mock;
import org.mockito.Mockito;
import org.mockito.MockitoAnnotations;
import org.springframework.boot.test.mock.mockito.MockBean;
import org.springframework.data.domain.PageImpl;
import org.springframework.data.domain.Pageable;

public abstract class MovieTestBase {

    @Mock
    protected MovieRepository repository;

    @MockBean
    protected MovieService service;

    @BeforeEach
    public void setup() {
        MockitoAnnotations.openMocks(this);
        service = new MovieService(repository);
        this.mockRepositoryMethods();
    }

    private void mockRepositoryMethods() {
        Mockito
            .when(repository.findById(Mockito.anyLong()))
            .thenThrow(ResourceNotFoundException.class);
        Mockito
            .when(
                repository.findById(
                    MockedEntities.PIRATES_OF_THE_CARIBBEAN.getId()
                )
            )
            .thenReturn(Optional.of(MockedEntities.PIRATES_OF_THE_CARIBBEAN));

        Mockito
            .when(repository.findAll(Mockito.any(Pageable.class)))
            .thenReturn(new PageImpl<>(MockedEntities.MOVIES));
        Mockito
            .doReturn(new PageImpl<>(List.of()))
            .when(repository)
            .findByTitleContainingAndDirector_SurnameContainingAndActors_SurnameContaining(
                Mockito.any(Pageable.class),
                Mockito.anyString(),
                Mockito.anyString(),
                Mockito.anyString()
            );
        Mockito
            .doReturn(
                new PageImpl<>(List.of(MockedEntities.PIRATES_OF_THE_CARIBBEAN))
            )
            .when(repository)
            .findByTitleContainingAndDirector_SurnameContainingAndActors_SurnameContaining(
                Mockito.any(Pageable.class),
                Mockito.contains(
                    MockedEntities.PIRATES_OF_THE_CARIBBEAN
                        .getTitle()
                        .substring(2, 8)
                ),
                Mockito.anyString(),
                Mockito.anyString()
            );
        Mockito
            .doReturn(
                new PageImpl<>(
                    List.of(
                        MockedEntities.PIRATES_OF_THE_CARIBBEAN,
                        MockedEntities.RANGO
                    )
                )
            )
            .when(repository)
            .findByTitleContainingAndDirector_SurnameContainingAndActors_SurnameContaining(
                Mockito.any(Pageable.class),
                Mockito.anyString(),
                Mockito.contains(
                    MockedEntities.GORE_VERBINSKI.getSurname().substring(2, 8)
                ),
                Mockito.anyString()
            );
        Mockito
            .doReturn(
                new PageImpl<>(
                    List.of(
                        MockedEntities.PIRATES_OF_THE_CARIBBEAN,
                        MockedEntities.RANGO
                    )
                )
            )
            .when(repository)
            .findByTitleContainingAndDirector_SurnameContainingAndActors_SurnameContaining(
                Mockito.any(Pageable.class),
                Mockito.anyString(),
                Mockito.anyString(),
                Mockito.contains(
                    MockedEntities.JOHNNY_DEPP.getSurname().substring(2, 4)
                )
            );
        Mockito
            .when(repository.save(MockedEntities.PIRATES_OF_THE_CARIBBEAN))
            .thenReturn(MockedEntities.PIRATES_OF_THE_CARIBBEAN);
    }
}
