package cz.muni.pa165.rating;

import cz.muni.pa165.core.exception.ResourceNotFoundException;
import cz.muni.pa165.rating.data.model.RatingFilter;
import cz.muni.pa165.rating.data.repository.MovieRatingRepository;
import cz.muni.pa165.rating.service.MovieRatingService;
import java.util.Optional;
import org.junit.jupiter.api.BeforeEach;
import org.mockito.Mock;
import org.mockito.Mockito;
import org.mockito.MockitoAnnotations;
import org.springframework.boot.test.mock.mockito.MockBean;
import org.springframework.data.domain.PageImpl;
import org.springframework.data.domain.Pageable;

public abstract class MovieRatingTestBase {

    @Mock
    protected MovieRatingRepository repository;

    @MockBean
    protected MovieRatingService service;

    @BeforeEach
    public void setup() {
        MockitoAnnotations.openMocks(this);
        this.mockRepositoryMethods();
        service = new MovieRatingService(repository);
    }

    private void mockRepositoryMethods() {
        Mockito
            .doThrow(ResourceNotFoundException.class)
            .when(repository)
            .findById(Mockito.anyLong());
        Mockito
            .doReturn(Optional.of(MockedEntities.RATING_1))
            .when(repository)
            .findById(MockedEntities.RATING_1.getId());
        Mockito
            .doReturn(new PageImpl<>(MockedEntities.RATINGS))
            .when(repository)
            .findAll(Mockito.any(Pageable.class));
        Mockito
            .doReturn(new PageImpl<>(MockedEntities.RATINGS))
            .when(repository)
            .findAllFiltered(
                Mockito.any(RatingFilter.class),
                Mockito.any(Pageable.class)
            );
        Mockito
            .doReturn(MockedEntities.RATING_1)
            .when(repository)
            .save(MockedEntities.RATING_1);
    }
}
