package cz.muni.pa165.rating.service;

import cz.muni.pa165.core.exception.ResourceNotFoundException;
import cz.muni.pa165.rating.MockedEntities;
import cz.muni.pa165.rating.MovieRatingTestBase;
import cz.muni.pa165.rating.data.model.RatingFilter;
import org.junit.jupiter.api.Assertions;
import org.junit.jupiter.api.Test;
import org.mockito.Mockito;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.data.domain.PageImpl;
import org.springframework.data.domain.PageRequest;

@SpringBootTest
class MovieRatingServiceTest extends MovieRatingTestBase {

    @Test
    void findByIdExisting() {
        Assertions.assertSame(
            MockedEntities.RATING_1,
            service.findById(MockedEntities.RATING_1.getId())
        );
    }

    @Test
    void findByIdNotExisting() {
        Assertions.assertThrows(
            ResourceNotFoundException.class,
            () -> service.findById(5L)
        );
    }

    @Test
    void findAll() {
        Assertions.assertEquals(
            new PageImpl<>(MockedEntities.RATINGS),
            service.findAll(new RatingFilter(), PageRequest.of(0, 5))
        );
    }

    @Test
    void create() {
        Assertions.assertSame(
            MockedEntities.RATING_1,
            service.create(MockedEntities.RATING_1)
        );
    }

    @Test
    void update() {
        Assertions.assertSame(
            MockedEntities.RATING_1,
            service.update(MockedEntities.RATING_1)
        );
        Mockito.verify(repository).findById(MockedEntities.RATING_1.getId());
    }

    @Test
    void delete() {
        service.delete(MockedEntities.RATING_1);
        Mockito.verify(repository).delete(MockedEntities.RATING_1);
    }
}
